@@ -10,12 +10,13 @@ import uuid
 
 import arrow
 import flask
+import markupsafe
 import slugify
 from pony import orm
 from werkzeug.utils import cached_property
 
-from . import (caching, cards, html_entry, links, markdown, model, path_alias,
-               queries, tokens, user, utils)
+from . import (caching, cards, html_entry, image, links, markdown, model,
+               path_alias, queries, tokens, user, utils)
 from .config import config
 from .utils import CallableProxy, CallableValue, TrueCallableProxy
 
@@ -300,7 +301,8 @@ class Entry(caching.Memoizable):
         if not self.authorized:
             return '', '', False
 
-        body, _, more = self._message.get_payload().partition('\n.....\n')
+        payload = typing.cast(str, self._message.get_payload())
+        body, _, more = payload.partition('\n.....\n')
         if not more and body.startswith('.....\n'):
             # The entry began with a cut, which failed to parse.
             # This rule is easier/faster than dealing with a regex from
@@ -311,7 +313,7 @@ class Entry(caching.Memoizable):
         _, ext = os.path.splitext(self._record.file_path)
         is_markdown = ext == '.md'
 
-        return body, more, is_markdown
+        return body.strip(), more.strip(), is_markdown
 
     @cached_property
     def body(self) -> typing.Callable[..., str]:
@@ -409,7 +411,7 @@ class Entry(caching.Memoizable):
 
             def og_tag(key, val) -> str:
                 """ produce an OpenGraph tag with the given key and value """
-                return utils.make_tag('meta', {'property': key, 'content': val}, start_end=True)
+                return utils.make_tag('meta', {'property': key, 'content': val})
 
             tags = og_tag('og:title', self.title(markup=False))
             tags += og_tag('og:url', self.link(absolute=True))
@@ -426,8 +428,8 @@ class Entry(caching.Memoizable):
                                          counter=markdown.ItemCounter())
             card = cards.extract_card(html_text)
 
-            for (image, width, height) in card.images[:kwargs.get('count', 1)]:
-                tags += og_tag('og:image', image)
+            for (img, width, height) in card.images[:kwargs.get('count', 1)]:
+                tags += og_tag('og:image', img)
                 if width:
                     tags += og_tag('og:image:width', width)
                 if height:
@@ -437,7 +439,7 @@ class Entry(caching.Memoizable):
             if description:
                 tags += og_tag('og:description', description)
 
-            return flask.Markup(tags)
+            return markupsafe.Markup(tags)
 
         return CallableProxy(_get_card)
 
@@ -480,7 +482,7 @@ class Entry(caching.Memoizable):
                                              counter=markdown.ItemCounter())
                 html_text = html_entry.first_paragraph(html_text)
                 if markup:
-                    return flask.Markup(html_text)
+                    return markupsafe.Markup(html_text)
 
                 return html_entry.strip_html(
                     html_text,
@@ -493,13 +495,15 @@ class Entry(caching.Memoizable):
     @cached_property
     def last_modified(self) -> arrow.Arrow:
         """ Get the date of last file modification """
-        if self.get('Last-Modified'):
-            return arrow.get(self.get('Last-Modified'))
+        if 'Last-Modified' in self._message:
+            return arrow.get(self._message['Last-Modified'])
         return self.date
 
     @property
     def authorized(self) -> bool:
         """ Returns if the current user is authorized to see this entry """
+        if self._record.auth:
+            flask.g.user_dependent = True
         return self._record.is_authorized(user.get_active())
 
     def _get_markup(self, text, is_markdown, args,
@@ -512,7 +516,7 @@ class Entry(caching.Memoizable):
             is_markdown -- whether the entry is formatted as Markdown
             kwargs -- parameters to pass to the Markdown processor
         """
-        # pylint:disable=too-many-arguments
+        # pylint:disable=too-many-arguments,too-many-positional-arguments
         if is_markdown:
             # Set defaults for the ID link generators, so permalinks from category
             # pages work correctly
@@ -589,7 +593,7 @@ class Entry(caching.Memoizable):
                              counter=counter)
 
         if footnotes:
-            return flask.Markup(f"<ol>{''.join(footnotes)}</ol>")
+            return markupsafe.Markup(f"<ol>{''.join(footnotes)}</ol>")
         return ''
 
     def _get_toc(self, body, more, max_depth, args) -> str:
@@ -604,7 +608,7 @@ class Entry(caching.Memoizable):
             self._get_markup(more, True, args=args, toc_buffer=tocs, counter=counter)
 
         if tocs:
-            return flask.Markup(markdown.toc_to_html(tocs, max_depth))
+            return markupsafe.Markup(markdown.toc_to_html(tocs, max_depth))
         return ''
 
     def _get_counter(self, section, args) -> markdown.ItemCounter:
@@ -641,6 +645,16 @@ class Entry(caching.Memoizable):
         """ Return whether an attribute is authorized to be read """
         return name.lower() in ('uuid', 'id', 'date', 'last-modified') or self.authorized
 
+    def __contains__(self, name):
+        """ Proxy undefined properties to the backing objects """
+        if not self._authorized_attr(name):
+            return False
+
+        if name.lower() not in ('auth') and hasattr(self._record, name):
+            return True
+
+        return name in self._message
+
     def __getattr__(self, name):
         """ Proxy undefined properties to the backing objects """
 
@@ -686,21 +700,43 @@ class Entry(caching.Memoizable):
 
         result: typing.List[Entry] = []
         cur_user = user.get_active()
-        for record in entries:
-            if count is not None and len(result) >= count:
+
+        # without a known limit, pony just fetches the entire freaking table,
+        # so we need to retrieve chunk-wise
+        start = 0
+        if count is None:
+            # We're going to do a full query retrieval (either due to lack of
+            # view constraint or using a date-based constraint), so we'd might
+            # as well just go for it instead of trying to be too fancy with
+            # clever loop conditionals
+            count = len(entries)
+        chunk_size = max(16, count)
+
+        while len(result) < count:
+            chunk = entries[start:start + chunk_size]
+            start += chunk_size
+
+            if not chunk:
                 break
 
-            auth = record.is_authorized(cur_user)
-            if auth or unauthorized:
-                result.append(Entry.load(record))
-                if not auth and unauthorized is not True:
-                    unauthorized -= 1
+            for record in chunk:
+                if len(result) >= count:
+                    break
 
-            if not auth:
-                tokens.request(cur_user)
+                auth = record.is_authorized(cur_user)
+                if auth or unauthorized:
+                    result.append(Entry.load(record))
+                    if not auth and unauthorized is not True:
+                        unauthorized -= 1
+                if not auth:
+                    tokens.request(cur_user)
 
         return result
 
+    def image(self, filename):
+        """ Retrieve an image using our search path as the context """
+        return image.get_image(filename, self.search_path)
+
 
 def get_entry_id(entry, fullpath, assign_id) -> typing.Optional[int]:
     """ Get or generate an entry ID for an entry """
@@ -814,7 +850,7 @@ def scan_file(fullpath: str, relpath: typing.Optional[str], fixup_pass: int) ->
 
     title = entry.get('title', '')
 
-    values = {
+    values: typing.Dict[str, typing.Any] = {
         'file_path': fullpath,
         'category': entry.get('Category', utils.get_category(relpath)),
         'status': model.PublishStatus[entry.get('Status', 'SCHEDULED').upper()].value,
@@ -825,7 +861,8 @@ def scan_file(fullpath: str, relpath: typing.Optional[str], fixup_pass: int) ->
         'redirect_url': entry.get('Redirect-To', ''),
         'title': title,
         'sort_title': entry.get('Sort-Title', title),
-        'canonical_path': entry.get('Path-Canonical', '')
+        'canonical_path': entry.get('Path-Canonical', ''),
+        'auth': entry.get('Auth', '')
     }
 
     entry_date = None
@@ -897,15 +934,6 @@ def scan_file(fullpath: str, relpath: typing.Optional[str], fixup_pass: int) ->
         for alias in entry.get_all('Path-Canonical', []):
             path_alias.set_alias(alias, model.AliasType.MOUNT, entry=record)
 
-    orm.delete(p for p in model.EntryAuth if p.entry == record)  # type:ignore
-    orm.commit()
-    for order, user_group in enumerate(entry.get('Auth', '').split()):
-        allowed = user_group[0] != '!'
-        if not allowed:
-            user_group = user_group[1:]
-        model.EntryAuth(order=order, entry=record, user_group=user_group, allowed=allowed)
-    orm.commit()
-
     with orm.db_session:
         set_tags = {
             utils.tag_key(t[0]): t
@@ -1004,18 +1032,30 @@ def scan_file(fullpath: str, relpath: typing.Optional[str], fixup_pass: int) ->
 @orm.db_session
 def expire_record(record):
     """ Expire a record for a missing entry """
+    from .flask_wrapper import current_app
 
     # This entry no longer exists so delete anything that relies on it
     orm.delete(pa for pa in model.PathAlias if pa.entry == record)
 
     # mark the entry as GONE to remove it from indexes
     record.status = model.PublishStatus.GONE.value
+
+    # remove it from full-text search
+    current_app.search_index.remove(record.id)
+
     orm.commit()
 
 
 @orm.db_session
 def remove_by_path(fullpath: str, entry_id: int):
     """ Remove entries for a path that don't match the expected ID """
+    from .flask_wrapper import current_app
+
+    # remove fulltext search entries
+    for fte in orm.select(e for e in model.Entry  # type:ignore
+                          if e.file_path == fullpath
+                          and e.id != entry_id):
+        current_app.search_index.remove(fte.id)
 
     orm.delete(pa for pa in model.PathAlias  # type:ignore
                if pa.entry.file_path == fullpath
@@ -1024,3 +1064,16 @@ def remove_by_path(fullpath: str, entry_id: int):
                if e.file_path == fullpath
                and e.id != entry_id)
     orm.commit()
+
+
+def get_entry_by_id(entry_id: int):
+    """ Given an entry ID, return the entry object """
+
+    record = model.Entry.get(id=entry_id)
+    if not record.visible:
+        return None
+
+    if not record.is_authorized(user.get_active()):
+        return None
+
+    return Entry.load(record)
