@@ -10,6 +10,7 @@ import urllib.parse
 from typing import Optional
 
 import flask
+import markupsafe
 import misaka
 import pygments
 import pygments.formatters
@@ -28,7 +29,7 @@ TocBuffer = typing.List[TocEntry]
 TOC_ALLOWED_TAGS = ('sup', 'sub',
                     'em', 'strong',
                     'b', 'i',
-                    'code',
+                    'code', 'tt',
                     'del', 'add', 'mark')
 
 # Remove these tags from plaintext-style conversions
@@ -138,7 +139,7 @@ class HtmlRenderer(misaka.HtmlRenderer):
                  footnote_buffer: typing.Optional[typing.List[str]],
                  toc_buffer: typing.Optional[TocBuffer],
                  counter: ItemCounter):
-        # pylint:disable=no-member,too-many-arguments
+        # pylint:disable=no-member,too-many-arguments,too-many-positional-arguments
         super().__init__(0, args.get('xhtml') and misaka.HTML_USE_XHTML or 0)
 
         self._config = args
@@ -357,7 +358,7 @@ class HtmlRenderer(misaka.HtmlRenderer):
                         'href': container_args['more_link'],
                         'class': container_args.get('more_class', False)
                     }))
-            text += flask.Markup(more_text)
+            text += markupsafe.Markup(more_text)
 
         if text and (container_args.get('div_class') or
                      container_args.get('div_style')):
@@ -496,9 +497,9 @@ class HtmlRenderer(misaka.HtmlRenderer):
             path, image_args, title = image.parse_image_spec(spec)
         except Exception as err:  # pylint: disable=broad-except
             LOGGER.exception("Got error on spec %s: %s", spec, err)
-            return flask.Markup('<span class="error">Couldn\'t parse image spec: ' +
-                                '<code>{}</code> {}</span>'.format(flask.escape(spec),
-                                                                   flask.escape(str(err))))
+            return markupsafe.Markup('<span class="error">Couldn\'t parse image spec: ' +
+                                     '<code>{}</code> {}</span>'.format(flask.escape(spec),
+                                                                        flask.escape(str(err))))
 
         composite_args = {**container_args, **image_args}
 
@@ -511,7 +512,7 @@ class HtmlRenderer(misaka.HtmlRenderer):
                                    _mark_rewritten=True)
         except Exception as err:  # pylint: disable=broad-except
             LOGGER.exception("Got error on image %s: %s", path, err)
-            return flask.Markup('<span class="error">Error loading image {}: {}</span>'.format(
+            return markupsafe.Markup('<span class="error">Error loading image {}: {}</span>'.format(
                 flask.escape(spec), flask.escape(str(err))))
 
 
@@ -530,7 +531,7 @@ def to_html(text, args, search_path,
 
     postprocess -- whether to postprocess the buffers for smartypants/HTML/etc.
     """
-    # pylint:disable=too-many-arguments
+    # pylint:disable=too-many-arguments,too-many-positional-arguments
 
     LOGGER.debug("counter: %s %s", id(counter), counter)
 
@@ -566,7 +567,7 @@ def to_html(text, args, search_path,
         footnotes[:] = (html_entry.process(text, args, search_path)
                         for text in footnotes)
 
-    return flask.Markup(text)
+    return markupsafe.Markup(text)
 
 
 def get_counters(text, args):
@@ -625,7 +626,7 @@ def render_title(text, markup=True, smartquotes=True, markdown_extensions=None):
         text = misaka.smartypants(text)
 
     if markup:
-        return flask.Markup(text)
+        return markupsafe.Markup(text)
 
     return html_entry.strip_html(text, remove_elements=PLAINTEXT_REMOVE_ELEMENTS)
 
