@@ -12,8 +12,8 @@ import werkzeug.exceptions as http_error
 from flask import redirect, request, send_file, url_for
 from pony import orm
 
-from . import (__version__, caching, image, index, model, path_alias, queries,
-               user, utils, view)
+from . import (caching, image, index, model, path_alias, queries, user, utils,
+               view)
 from .caching import cache
 from .category import Category
 from .config import config
@@ -23,15 +23,6 @@ from .template import Template, map_template
 
 LOGGER = logging.getLogger(__name__)
 
-# mapping from template extension to MIME type; probably could be better
-EXTENSION_MAP = {
-    '.html': 'text/html; charset=utf-8',
-    '.xml': 'application/xml',
-    '.json': 'application/json',
-    '.css': 'text/css; charset=utf-8',
-    '.txt': 'text/plain; charset=utf-8'
-}
-
 # Headers for responses that shouldn't be cached
 NO_CACHE = {
     'Cache-control': 'private, no-cache, no-store, max-age=0',
@@ -39,10 +30,22 @@ NO_CACHE = {
 }
 
 
+def cache_control():
+    """ Determine the cache-control value based on page rendering """
+    if flask.g.get('user_dependent'):
+        # page had a user dependency, so don't cache it
+        return 'private, no-cache'
+
+    # page did not have a user dependency, so it's cacheable
+    timeout = config.cache.get('CACHE_DEFAULT_TIMEOUT', None)
+    if timeout:
+        return f'public, max-age={timeout}'
+    return 'public'
+
+
 def mime_type(template: Template) -> str:
     """ infer the content-type from the extension """
-    _, ext = os.path.splitext(template.filename)
-    return EXTENSION_MAP.get(ext, 'text/html; charset=utf-8')
+    return f'{template.mime_type}; charset=utf-8'
 
 
 def get_template(template: str, relation) -> typing.Optional[str]:
@@ -94,7 +97,7 @@ def image_function(template=None,
     return lambda filename: image.get_image(filename, path)
 
 
-def render_publ_template(template: Template, **kwargs) -> typing.Tuple[str, str]:
+def render_publ_template(template: Template, is_error=True, **kwargs) -> typing.Tuple[str, str]:
     """ Render out a template, providing the image function based on the args.
 
     Returns tuple of (rendered text, etag)
@@ -102,7 +105,7 @@ def render_publ_template(template: Template, **kwargs) -> typing.Tuple[str, str]
     @cache.memoize(unless=caching.do_not_cache)
     def do_render(template: Template, **kwargs) -> typing.Tuple[str, str, typing.Dict]:
         LOGGER.debug("Rendering template %s with args %s and kwargs %s; caching=%s",
-                     template, request.args, kwargs, not caching.do_not_cache)
+                     template, request.args, kwargs, not caching.do_not_cache())
 
         args = {
             'template': template,
@@ -128,7 +131,14 @@ def render_publ_template(template: Template, **kwargs) -> typing.Tuple[str, str]
         return None
 
     try:
-        cur_user = user.get_active()
+        from . import __version__
+        try:
+            cur_user = user.get_active()
+        except http_error.HTTPException:
+            if not is_error:
+                raise
+            cur_user = None
+
         text, etag, flask.g.needs_auth = do_render(  # pylint:disable=assigning-non-slot
             template,
             user=cur_user,
@@ -136,7 +146,8 @@ def render_publ_template(template: Template, **kwargs) -> typing.Tuple[str, str]
             _url=request.url,
             _index_time=index.last_indexed(),
             _latest=latest_entry(),
-            _publ_version=__version__.__version__,
+            _publ_version=__version__,
+            _accept_mime=flask.request.accept_mimetypes,
             **kwargs)
         return text, etag
     except queries.InvalidQueryError as err:
@@ -175,10 +186,11 @@ def render_error(category, error_message, error_codes,
     template_list.append(str(int(error_code / 100) * 100))
     template_list.append('error')
 
-    template = map_template(category, template_list)
+    template = map_template(category, template_list, in_exception=True)
     if template:
         return render_publ_template(
             template,
+            is_error=True,
             category=Category.load(category),
             error={'code': error_code, 'message': error_message},
             exception=exception)[0], error_code, headers
@@ -291,17 +303,17 @@ def render_category_path(category: str, template: typing.Optional[str]):
                                   and e.visible).exists():
             raise http_error.NotFound("No such category")
 
-    if not template:
-        template = Category.load(category).get('Index-Template') or 'index'
+    template_name: str = template or Category.load(category).index_template
 
-    tmpl = map_template(category, template)
-
-    if not tmpl:
+    template_impl = map_template(category, template_name)
+    if not template_impl:
         # this might actually be a malformed category URL
-        test_path = '/'.join((category, template)) if category else template
+        test_path = '/'.join((category, template_name)) if category else template_name
         LOGGER.debug("Checking for malformed category %s", test_path)
-        record = model.Entry.select(lambda e: e.category ==
-                                    test_path and e.visible).exists()  # type:ignore
+        record = model.Entry.select(
+            lambda e: (e.category == test_path
+                       or e.category.startswith(f'{test_path}/'))
+            and e.visible).exists()  # type:ignore
         if record:
             LOGGER.debug("Redirecting to category %s; request.args=%s", test_path, request.args)
             return redirect(url_for('category', category=test_path,
@@ -320,15 +332,16 @@ def render_category_path(category: str, template: typing.Optional[str]):
         raise http_error.BadRequest(str(err))
 
     rendered, etag = render_publ_template(
-        tmpl,
+        template_impl,
         category=Category.load(category),
         view=view_obj)
 
     if request.if_none_match.contains(etag):
         return 'Not modified', 304, {'ETag': f'"{etag}"'}
 
-    return rendered, {'Content-Type': mime_type(tmpl),
-                      'ETag': f'"{etag}"'}
+    return rendered, {'Content-Type': mime_type(template_impl),
+                      'ETag': f'"{etag}"',
+                      'Cache-Control': cache_control()}
 
 
 @ orm.db_session
@@ -518,7 +531,8 @@ def render_entry_record(record: model.Entry, category: str, template: typing.Opt
 
     headers = {
         'Content-Type': entry_obj.get('Content-Type', mime_type(tmpl)),
-        'ETag': etag
+        'ETag': etag,
+        'Cache-Control': cache_control()
     }
 
     return rendered, headers
