@@ -8,10 +8,13 @@ import arrow
 import flask
 from werkzeug.utils import cached_property
 
-from . import caching, queries, tokens, user, utils
+from . import caching
+from . import category as publ_category
+from . import queries, tokens, user, utils
 from .entry import Entry
 
 ViewSpec = typing.Dict[str, typing.Any]  # pylint:disable=invalid-name
+PaginationType = typing.Union[utils.TimeSpan, typing.Literal['count'], None]
 
 LOGGER = logging.getLogger(__name__)
 
@@ -91,15 +94,18 @@ class View(caching.Memoizable):
 
         count = spec.pop('count', None)
 
+        self._deleted = queries.build_query({
+            **spec,
+            '_deleted': True
+        }).order_by(*queries.ORDER_BY[self._order_by])
         self._entries = queries.build_query(
             spec).order_by(*queries.ORDER_BY[self._order_by])
 
+        self.type: PaginationType = None
         if self.spec.get('date') is not None:
             _, self.type, _ = utils.parse_date(self.spec['date'])
         elif count is not None:
             self.type = 'count'
-        else:
-            self.type = ''
 
     def _key(self):
         return repr(self.spec)
@@ -129,56 +135,97 @@ class View(caching.Memoizable):
 
         return utils.CallableProxy(_entries)
 
+    @cached_property
+    def has_unauthorized(self) -> bool:
+        """ Returns whether there's entries that the user could see if they
+        were authorized differently """
+
+        LOGGER.debug("HAS_UNAUTHORIZED")
+
+        count = self.spec.get('count')
+        cur_user = user.get_active()
+
+        if cur_user and cur_user.is_admin:
+            return False
+
+        auth_count = 0
+
+        LOGGER.debug("Checking for unauthorized in the first %s entries", count)
+
+        # retrieve entries a chunk at a time so Pony doesn't have to realize
+        # the whole table up-front
+        start = 0
+        chunk_size = max(count if count is not None else 0, 16)
+        while count is None or auth_count < count:
+            LOGGER.debug("Testing %d", start)
+            chunk = self._entries[start:start + chunk_size]
+            start += chunk_size
+
+            if not chunk:
+                LOGGER.debug("Hit EOF")
+                break
+
+            for record in chunk:
+                if not record.is_authorized(cur_user):
+                    LOGGER.debug("Found unauthorized entry")
+                    tokens.request(cur_user)
+                    return True
+                auth_count += 1
+                LOGGER.debug("Seen %d authorized", auth_count)
+
+        return False
+
     @cached_property
     def unauthorized(self) -> typing.Callable[..., typing.List[Entry]]:
-        """ Gets entries which the user is not allowed to view """
+        """ Gets the entries which would be visible on the view if the user
+        were authorized to see them """
 
         def _unauthorized(count=None) -> typing.List[Entry]:
             result: typing.List[Entry] = []
-            if count is None:
-                count = self.spec.get('count')
 
+            auth_max = self.spec.get('count')
             cur_user = user.get_active()
-            for record in self._entries:
-                if count is not None and len(result) >= count:
+
+            if cur_user and cur_user.is_admin:
+                return result
+
+            auth_count = 0
+            start = 0
+            chunk_size = max(auth_max if auth_max is not None else 0, 16)
+            while ((auth_max is None or auth_count < auth_max)
+                   and (count is None or len(result) < count)):
+
+                chunk = self._entries[start:start + chunk_size]
+                start += chunk_size
+
+                if not chunk:
                     break
 
-                if not record.is_authorized(cur_user):
-                    tokens.request(cur_user)
-                    result.append(Entry.load(record))
+                for record in chunk:
+                    # if we've traversed the whole authorized count, exit
+                    if auth_max is not None and auth_count >= auth_max:
+                        break
+
+                    # if we've reached our unauthorized max, exit
+                    if count is not None and len(result) >= count:
+                        break
+
+                    if not record.is_authorized(cur_user):
+                        tokens.request(cur_user)
+                        result.append(Entry.load(record))
+                    else:
+                        auth_count += 1
 
             return result
 
         return utils.CallableProxy(_unauthorized)
 
-    @cached_property
-    def has_unauthorized(self) -> bool:
-        """ Returns whether there's entries that the user could see if they
-        were authorized differently """
-
-        count = self.spec.get('count')
-        auth_count = 0
-        cur_user = user.get_active()
-        for record in self._entries:
-            if count is not None and auth_count >= count:
-                # we've already hit the end of the list
-                break
-
-            if not record.is_authorized(cur_user):
-                # there's at least one unauthorized, so fast quit
-                tokens.request(cur_user)
-                return True
-
-            auth_count += 1
-
-        return False
-
     @cached_property
     def deleted(self) -> typing.List[Entry]:
         """ Gets the deleted entries from the view """
-        query = queries.build_query({**self.query_spec,
-                                     'future': False,
-                                     '_deleted': True})
+
+        query = self._deleted
+
         if self.spec.get('count'):
             # We're using a count constraint, so we'll want to limit this to
             # entries that come before our next page of non-deleted results.
@@ -200,7 +247,7 @@ class View(caching.Memoizable):
         return len(self.entries())
 
     @cached_property
-    def last_modified(self) -> typing.Optional[Entry]:
+    def last_modified(self) -> arrow.Arrow:
         """ Gets the most recent modification time for all entries in the view """
         entries = self.entries()
         if entries:
@@ -353,11 +400,15 @@ class View(caching.Memoizable):
             if taglist:
                 args['tag'] = list(utils.TagSet(utils.as_list(taglist)).keys())
 
+            view_category = publ_category.Category.load(category if category
+                                                        else self.spec.get('category'))
+
             return flask.url_for('category',
                                  **args,
-                                 template=template,
-                                 category=category if category else self.spec.get(
-                                     'category'),
+                                 template=template if str(template) != view_category.index_template
+                                 else '',
+                                 category=category if category
+                                 else self.spec.get('category'),
                                  _external=absolute,
                                  **kwargs)
 
